@@ -28,6 +28,11 @@
 typedef struct AvahiEntryGroup AvahiEntryGroup;
 typedef struct AvahiClient AvahiClient;
 
+namespace Gtk
+{
+  class Window;
+}
+
 namespace Glom
 {
 
@@ -135,7 +140,7 @@ public:
 
   /** Start a database server instance for the exisiting database files.
    */
-  void start_self_hosting();
+  bool start_self_hosting();
 
   /** Stop the database server instance for the database files.
    */
@@ -148,6 +153,19 @@ public:
   //Show the gda error in a dialog.
   static bool handle_error(bool cerr_only = false);
 
+
+  /** Check whether PostgreSQL is really available for self-hosting,
+   * in case the distro package has incorrect dependencies.
+   *
+   * @results True if everything is OK.
+   */
+  static bool check_postgres_is_available_with_warning();
+
+  /** Try to install postgres on the distro, though this will require a distro-specific 
+   * patch to the implementation.
+   */
+  static bool install_postgres(Gtk::Window* parent_window);
+
 protected:
   void on_sharedconnection_finished();
 
@@ -167,6 +185,8 @@ public:
   void avahi_create_services(AvahiClient *c);
 protected:
 
+  bool directory_exists(const std::string& uri);
+
   typedef std::list<Glib::ustring> type_list_ports;
   type_list_ports m_list_ports; //Network ports on which to try connecting to postgres.
 
